﻿
Namespace FileAccess

    ''' <summary>
    ''' Stellt zugriffe auf die Xml-Datei dar, die die Passwörter beinhaltet.
    ''' </summary>
    Public Class PasswordFile

        ''' <summary>
        ''' Gibt die Standard-Dateierweiterung an, die verwendet werden sollte (.xml).
        ''' </summary>
        Public Const FileExtension As String = ".xml"

        ''' <summary>
        ''' Lädt die Passwörter aus der angegebenen Datei und entschlüsselt sie mit dem angegebenen Master-Passwort.
        ''' </summary>
        ''' <param name="Path">Der Pfad der Datei, die die Passwörter beinhaltet.</param>
        ''' <param name="MainPassword">Das Master-Passwort.</param>
        Public Shared Function Load(Path As String, MainPassword As String) As IEnumerable(Of Password)
            Dim Document = XDocument.Load(Path)
            Dim RootElement = Document.Root
            Dim FileVersion = Version.Parse(RootElement.Attribute("Version").Value)
            Dim Accessor = FileAccessorBase.GetCompatibleAccessor(FileVersion)
            Return Accessor.XmlToPasswords(RootElement, MainPassword)
        End Function

        ''' <summary>
        ''' Verschlüsselt die angegebenen Passwörter mit dem angegebenen Master-Passwort und speichert sie in die angegebene Datei.
        ''' </summary>
        ''' <param name="Path">Der Pfad der Datei, in die geschrieben wird.</param>
        ''' <param name="MainPassword">Das Master-Passwort.</param>
        ''' <param name="Passwords">Die zu speichernden Passwörter.</param>
        Public Shared Sub Save(Path As String, MainPassword As String, Passwords As IEnumerable(Of Password))
            Dim Document As New XDocument
            Dim Accessor = FileAccessorBase.GetNewest
            Dim RootElement = Accessor.PasswordsToXml(Passwords, MainPassword)
            If RootElement.Attribute("Version") Is Nothing Then
                RootElement.SetAttributeValue("Version", Accessor.Version.ToString)
            End If
            Document.Add(RootElement)
            If Not System.IO.Directory.Exists(System.IO.Path.GetDirectoryName(Path)) Then
                System.IO.Directory.CreateDirectory(System.IO.Path.GetDirectoryName(Path))
            End If
            Document.Save(Path)
        End Sub

    End Class

End Namespace
