﻿
Namespace FileAccess

    ''' <summary>
    ''' Passwort -> UTF8 -> AddRandomness (Siehe <see cref="AccessorV10.AddRandomness"/>) -> AES -> Base64
    ''' AddRandomness fügt am Start und Ende je 20 bis 80 zufällige Bytes hinzu. Dadurch haben die verschlüsselten Daten nicht die gleiche Länge wie das Passwort.
    ''' Veraltet!
    ''' Durch das Hinzufügen von exakt 40 bis 160 zufälligen Bytes kann die Länge des Passwortes auf 0 bis n - 8 - 40 eingegrenzt werden (8 Bytes für die Größenangabe, 40 zufällige Zeichen). Die Wahrscheinlichkeit, dass wenig zufällige Zeichen bei einem kurzen Passwort hinzugefügt werden ist gering, aber trotzdem existent.
    ''' </summary>
    <Obsolete("Potenzielle Sicherheitslücke")>
    Public Class AccessorV10
        Inherits FileAccessorBase

        Public Overrides ReadOnly Property Version As System.Version
            Get
                Return New Version(1, 0)
            End Get
        End Property

        Public Overrides Function XmlToPasswords(RootElement As System.Xml.Linq.XElement, MainPassword As String) As System.Collections.Generic.IEnumerable(Of Password)
            Dim Temp As New List(Of Password)
            Dim Salt = Convert.FromBase64String(RootElement.Attribute("Salt").Value)
            Using Cryptor As New AesCryptor(MainPassword, Salt)
                For Each i In RootElement.Elements
                    Temp.Add(XmlToPassword(i, Cryptor))
                Next
            End Using
            Return Temp
        End Function

        Public Overrides Function PasswordsToXml(Passwords As System.Collections.Generic.IEnumerable(Of Password), MainPassword As String) As System.Xml.Linq.XElement
            Dim RootElement As New XElement("Passwords")
            Dim Salt = AesCryptor.GetRandomSalt
            RootElement.SetAttributeValue("Salt", Convert.ToBase64String(Salt))
            Using Cryptor As New AesCryptor(MainPassword, Salt)
                For Each i In Passwords
                    RootElement.Add(PasswordToXml(i, Cryptor))
                Next
            End Using
            Return RootElement
        End Function

        Private Shared Function PasswordToXml(Password As Password, Cryptor As AesCryptor) As XElement
            Static Encoding As New System.Text.UTF8Encoding(False)
            Dim PasswordElement As New XElement("Password")
            PasswordElement.SetAttributeValue("Password", Convert.ToBase64String(Cryptor.Encrypt(AddRandomness(Encoding.GetBytes(Password.Password)))))
            PasswordElement.SetAttributeValue("Description", Convert.ToBase64String(Cryptor.Encrypt(AddRandomness(Encoding.GetBytes(Password.Description)))))
            Return PasswordElement
        End Function

        Private Shared Function XmlToPassword(Element As XElement, Cryptor As AesCryptor) As Password
            Static Encoding As New System.Text.UTF8Encoding(False)
            Return New Password With {.Description = Encoding.GetString(RemoveRandomness(Cryptor.Decrypt(Convert.FromBase64String(Element.Attribute("Description").Value)))), _
                                      .Password = Encoding.GetString(RemoveRandomness(Cryptor.Decrypt(Convert.FromBase64String(Element.Attribute("Password").Value))))}
        End Function

        ''' <summary>
        ''' Fügt dem angegebenen Byte-Array vorne und hinten 20 bis 80 zufällige Bytes hinzu. Die Länge der zufälligen Daten wird in den ersten 8 Bytes (2x Int32) gespeichert.
        ''' Beispiel: Aus {1, 2, 3, 4, 5} wird
        ''' {
        '''   0, 0, 0, 10,    (10 Bytes vorne)
        '''   0, 0, 0, 8,    (8 Bytes hinten)
        '''   101, 102, 103, 104, 105, 106, 107, 108, 109, 110,    (10 zufällige Bytes vorne)
        '''   1, 2, 3, 4, 5,    (die 5 Bytes aus <paramref name="Payload"/>)
        '''   201, 202, 203, 204, 205, 206, 207, 208    (8 zufällige Bytes hinten)
        ''' }
        ''' </summary>
        ''' <param name="Payload">Die Bytes, denen zufällige Daten hinzugefügt werden.</param>
        Private Shared Function AddRandomness(Payload As Byte()) As Byte()
            Static Rnd As New Random

            Dim FrontLength = Rnd.Next(20, 81)
            Dim BackLength = Rnd.Next(20, 81)

            Dim Result = New Byte(4 + 4 + FrontLength + Payload.Length + BackLength - 1) {}
            Rnd.NextBytes(Result)

            Array.Copy(Int32ToBytes(FrontLength), 0, Result, 0, 4)
            Array.Copy(Int32ToBytes(BackLength), 0, Result, 4, 4)

            Array.Copy(Payload, 0, Result, 4 + 4 + FrontLength, Payload.Length)

            Return Result
        End Function

        ''' <summary>
        ''' Kehrt <see cref="AddRandomness"/> um.
        ''' </summary>
        Private Shared Function RemoveRandomness(Bytes As Byte()) As Byte()
            Dim FrontLength = BytesToInt32(Bytes.SubSequence(0, 4))
            Dim BackLength = BytesToInt32(Bytes.SubSequence(4, 4))
            Return Bytes.SubSequence(4 + 4 + FrontLength, Bytes.Length - 4 - 4 - FrontLength - BackLength)
        End Function

        Private Shared Function Int32ToBytes(Value As Int32) As Byte()
            Return {CByte((Value >> 24) And 255), _
                    CByte((Value >> 16) And 255), _
                    CByte((Value >> 8) And 255), _
                    CByte(Value And 255)}
        End Function

        Private Shared Function BytesToInt32(Bytes As Byte()) As Int32
            Return CInt(Bytes(0)) << 24 Or _
                   CInt(Bytes(1)) << 16 Or _
                   CInt(Bytes(2)) << 8 Or _
                   CInt(Bytes(3))
        End Function

    End Class

End Namespace
