﻿
Namespace FileAccess

    ''' <summary>
    ''' Passwort -> UTF8 -> AddPadding (Siehe <see cref="AccessorV11.AddPadding"/>) -> AES -> Base64
    ''' </summary>
    Public Class AccessorV11
        Inherits FileAccessorBase

        Public Overrides ReadOnly Property Version As System.Version
            Get
                Return New Version(1, 1)
            End Get
        End Property

        Public Overrides Function XmlToPasswords(RootElement As System.Xml.Linq.XElement, MainPassword As String) As System.Collections.Generic.IEnumerable(Of Password)
            Dim Temp As New List(Of Password)
            Dim Salt = Convert.FromBase64String(RootElement.Attribute("Salt").Value)
            Using Cryptor As New AesCryptor(MainPassword, Salt)
                For Each i In RootElement.Elements
                    Temp.Add(XmlToPassword(i, Cryptor))
                Next
            End Using
            Return Temp
        End Function

        Public Overrides Function PasswordsToXml(Passwords As System.Collections.Generic.IEnumerable(Of Password), MainPassword As String) As System.Xml.Linq.XElement
            Dim RootElement As New XElement("Passwords")
            Dim Salt = AesCryptor.GetRandomSalt
            RootElement.SetAttributeValue("Salt", Convert.ToBase64String(Salt))
            Using Cryptor As New AesCryptor(MainPassword, Salt)
                For Each i In Passwords
                    RootElement.Add(PasswordToXml(i, Cryptor))
                Next
            End Using
            Return RootElement
        End Function

        Private Shared Function PasswordToXml(Password As Password, Cryptor As AesCryptor) As XElement
            Static Encoding As New System.Text.UTF8Encoding(False)
            Dim PasswordElement As New XElement("Password")
            PasswordElement.SetAttributeValue("Password", Convert.ToBase64String(Cryptor.Encrypt(AddPadding(Encoding.GetBytes(Password.Password)))))
            PasswordElement.SetAttributeValue("Description", Convert.ToBase64String(Cryptor.Encrypt(AddPadding(Encoding.GetBytes(Password.Description)))))
            Return PasswordElement
        End Function

        Private Shared Function XmlToPassword(Element As XElement, Cryptor As AesCryptor) As Password
            Static Encoding As New System.Text.UTF8Encoding(False)
            Return New Password With {.Description = Encoding.GetString(RemovePadding(Cryptor.Decrypt(Convert.FromBase64String(Element.Attribute("Description").Value)))), _
                                      .Password = Encoding.GetString(RemovePadding(Cryptor.Decrypt(Convert.FromBase64String(Element.Attribute("Password").Value))))}
        End Function

        ''' <summary>
        ''' Füllt die Daten mit zufälligen Bytes auf das nächste oder übernächste (zufällig, 50% Wahrscheinlichkeit) Vielfache von 64 auf.
        ''' </summary>
        ''' <param name="Payload">Das Byte-Array, zu dem zufällige Daten hinzugefügt werden.</param>
        Private Shared Function AddPadding(Payload As Byte()) As Byte()
            Static Rnd As New Random

            Const PaddingBlockLength = 64

            Dim PaddingLength As Integer
            If Payload.Length = 0 Then
                PaddingLength = PaddingBlockLength
            Else
                'Die Länge wird auf das nächste vielfache von 64 Bytes aufgerundet.
                Dim ResultLength = (Payload.Length \ PaddingBlockLength + 1) * PaddingBlockLength
                PaddingLength = ResultLength - Payload.Length
            End If

            If Rnd.NextDouble < 0.5 Then
                PaddingLength += PaddingBlockLength
            End If

            Dim Result = New Byte(4 + Payload.Length + PaddingLength - 1) {}
            Rnd.NextBytes(Result)

            Array.Copy(Int32ToBytes(PaddingLength), 0, Result, 0, 4)
            If Not Payload.Length = 0 Then
                Array.Copy(Payload, 0, Result, 4, Payload.Length)
            End If

            Return Result
        End Function

        ''' <summary>
        ''' Kehrt <see cref="AddPadding"/> um.
        ''' </summary>
        Private Shared Function RemovePadding(Bytes As Byte()) As Byte()
            Dim PaddingLength = BytesToInt32(Bytes.SubSequence(0, 4))
            Return Bytes.SubSequence(4, Bytes.Length - 4 - PaddingLength)
        End Function

        Private Shared Function Int32ToBytes(Value As Int32) As Byte()
            Return {CByte((Value >> 24) And 255), _
                    CByte((Value >> 16) And 255), _
                    CByte((Value >> 8) And 255), _
                    CByte(Value And 255)}
        End Function

        Private Shared Function BytesToInt32(Bytes As Byte()) As Int32
            Return CInt(Bytes(0)) << 24 Or _
                   CInt(Bytes(1)) << 16 Or _
                   CInt(Bytes(2)) << 8 Or _
                   CInt(Bytes(3))
        End Function

    End Class

End Namespace
