﻿Public Class GeneratePasswordDialog

    Private Shared ReadOnly UpperCaseLetters As Char() = "ABCDEFGHIJKLMNOPQRSTUVWXYZ".ToCharArray
    Private Shared ReadOnly LowerCaseLetters As Char() = "abcdefghijklmnopqrstuvwxyz".ToCharArray
    Private Shared ReadOnly Digits As Char() = "0123456789".ToCharArray
    Private Shared ReadOnly SpecialChars As Char() = " !""§$%&/()=?`ß´{[]}\+#-.,*'_:;~<>|@€µ²³^°".ToCharArray

    Dim _Password As String
    Public ReadOnly Property Password As String
        Get
            Return _Password
        End Get
    End Property

    Dim WithEvents ViewModel As GeneratePasswordDialogViewModel
    Dim KeySpace As New List(Of Char())
    Dim KeySpaceLength As Integer = 0
    Dim Rnd As New Random

    Public Sub New()
        InitializeComponent()
        ViewModel = DirectCast(Me.DataContext, GeneratePasswordDialogViewModel)
        UpdateKeySpace()
    End Sub

    Private Sub CloseOk(sender As System.Object, e As System.Windows.RoutedEventArgs)
        Me.DialogResult = True
        Me.Close()
    End Sub

    Private Sub CloseCancel(sender As System.Object, e As System.Windows.RoutedEventArgs)
        Me.DialogResult = False
        Me.Close()
    End Sub

    Private Function CreatePassword() As String
        If KeySpace.Count = 0 Then
            Throw New NopeException
        End If
        Dim Chars = New Char(ViewModel.PasswordLength - 1) {}
        Dim UsedKeySpaceParts As Boolean()
        Do
            UsedKeySpaceParts = New Boolean(KeySpace.Count - 1) {}
            For i = 0 To Chars.Length - 1
                Dim KeySpaceIndex = Rnd.Next(0, KeySpaceLength)
                For j = 0 To KeySpace.Count - 1
                    If KeySpaceIndex >= KeySpace(j).Length Then
                        KeySpaceIndex -= KeySpace(j).Length
                    Else
                        Chars(i) = KeySpace(j)(KeySpaceIndex)
                        UsedKeySpaceParts(j) = True
                        Exit For
                    End If
                Next
            Next
        Loop Until UsedKeySpaceParts.All
        Return New String(Chars)
    End Function

    Private Sub ViewModel_KeySpaceChanged() Handles ViewModel.KeySpaceChanged
        UpdateKeySpace()
    End Sub

    Private Sub ViewModel_LengthChanged() Handles ViewModel.LengthChanged
        UpdatePassword()
    End Sub

    Private Sub UpdateKeySpace()
        KeySpace.Clear()
        KeySpaceLength = 0
        If ViewModel.KeySpaceIsValid Then
            If ViewModel.UseUpperCaseLetters Then
                KeySpace.Add(UpperCaseLetters)
                KeySpaceLength += UpperCaseLetters.Length
            End If
            If ViewModel.UseLowerCaseLetters Then
                KeySpace.Add(LowerCaseLetters)
                KeySpaceLength += LowerCaseLetters.Length
            End If
            If ViewModel.UseDigits Then
                KeySpace.Add(Digits)
                KeySpaceLength += Digits.Length
            End If
            If ViewModel.UseSpecialChars Then
                KeySpace.Add(SpecialChars)
                KeySpaceLength += SpecialChars.Length
            End If
        End If
        UpdatePassword()
    End Sub

    Private Sub UpdatePassword()
        If KeySpace.Count = 0 Then
            ViewModel.ExamplePassword = ""
            _Password = Nothing
        Else
            ViewModel.ExamplePassword = CreatePassword()
            _Password = CreatePassword()
        End If
    End Sub

End Class
