Public Class GeneratePasswordDialogViewModel
    Inherits ViewModelBase

    Public Const DefaultMaximumPasswordLength As Integer = 100

    Public Event KeySpaceChanged()
    Public Event LengthChanged()

    Dim _UseUpperCaseLetters As Boolean = True
    Public Property UseUpperCaseLetters As Boolean
        Get
            Return _UseUpperCaseLetters
        End Get
        Set(value As Boolean)
            If ChangeIfDifferent(_UseUpperCaseLetters, value, "UseUpperCaseLetters") Then
                UpdateMinimumPasswordLength()
                RaiseEvent KeySpaceChanged()
            End If
        End Set
    End Property

    Dim _UseLowerCaseLetters As Boolean = True
    Public Property UseLowerCaseLetters As Boolean
        Get
            Return _UseLowerCaseLetters
        End Get
        Set(value As Boolean)
            If ChangeIfDifferent(_UseLowerCaseLetters, value, "UseLowerCaseLetters") Then
                UpdateMinimumPasswordLength()
                RaiseEvent KeySpaceChanged()
            End If
        End Set
    End Property

    Dim _UseDigits As Boolean = True
    Public Property UseDigits As Boolean
        Get
            Return _UseDigits
        End Get
        Set(value As Boolean)
            If ChangeIfDifferent(_UseDigits, value, "UseDigits") Then
                UpdateMinimumPasswordLength()
                RaiseEvent KeySpaceChanged()
            End If
        End Set
    End Property

    Dim _UseSpecialChars As Boolean = True
    Public Property UseSpecialChars As Boolean
        Get
            Return _UseSpecialChars
        End Get
        Set(value As Boolean)
            If ChangeIfDifferent(_UseSpecialChars, value, "UseSpecialChars") Then
                UpdateMinimumPasswordLength()
                RaiseEvent KeySpaceChanged()
            End If
        End Set
    End Property

    <Dependency("UseUpperCaseLetters", "UseLowerCaseLetters", "UseDigits", "UseSpecialChars")>
    Public ReadOnly Property KeySpaceIsValid As Boolean
        Get
            Return _UseUpperCaseLetters OrElse _UseLowerCaseLetters OrElse _UseDigits OrElse _UseSpecialChars
        End Get
    End Property

    Dim _PasswordLengthText As String = "32"
    Public Property PasswordLengthText As String
        Get
            Return _PasswordLengthText
        End Get
        Set(value As String)
            If ChangeIfDifferent(_PasswordLengthText, value, "PasswordLengthText") Then
                Dim Temp As Integer
                PasswordLengthTextIsValid = Not String.IsNullOrEmpty(value) AndAlso Integer.TryParse(value, Temp) AndAlso Temp >= MinimumPasswordLength
                If _PasswordLengthTextIsValid Then
                    _PasswordLength = Temp
                    OnPropertyChanged("PasswordLength")
                    RaiseEvent LengthChanged()
                End If
            End If
        End Set
    End Property

    Dim _PasswordLengthTextIsValid As Boolean = True
    Public Property PasswordLengthTextIsValid As Boolean
        Get
            Return _PasswordLengthTextIsValid
        End Get
        Private Set(value As Boolean)
            ChangeIfDifferent(_PasswordLengthTextIsValid, value, "PasswordLengthTextIsValid")
        End Set
    End Property

    Dim _PasswordLength As Integer = 32
    Public Property PasswordLength As Integer
        Get
            Return _PasswordLength
        End Get
        Set(value As Integer)
            If ChangeIfDifferent(_PasswordLength, Math.Min(value, DefaultMaximumPasswordLength), "PasswordLength") Then
                _PasswordLengthText = _PasswordLength.ToString
                OnPropertyChanged("PasswordLengthText")
                PasswordLengthTextIsValid = True
                RaiseEvent LengthChanged()
            End If
        End Set
    End Property

    <Dependency("PasswordLength")>
    Public ReadOnly Property MaximumPasswordLength As Integer
        Get
            Return Math.Max(DefaultMaximumPasswordLength, _PasswordLength)
        End Get
    End Property

    Dim _MinimumPasswordLength As Integer
    Public Property MinimumPasswordLength As Integer
        Get
            Return _MinimumPasswordLength
        End Get
        Private Set(value As Integer)
            ChangeIfDifferent(_MinimumPasswordLength, value, "MinimumPasswordLength")
        End Set
    End Property

    Dim _ExamplePassword As String = ""
    Public Property ExamplePassword As String
        Get
            Return _ExamplePassword
        End Get
        Set(value As String)
            ChangeIfDifferent(_ExamplePassword, value, "ExamplePassword")
        End Set
    End Property

    Public Sub New()
        MyBase.New(True)
        UpdateMinimumPasswordLength()
    End Sub

    Private Sub UpdateMinimumPasswordLength()
        MinimumPasswordLength = Math.Max({_UseUpperCaseLetters, _UseLowerCaseLetters, _UseDigits, _UseSpecialChars}.Count(True), 1)
        If _PasswordLength < _MinimumPasswordLength Then
            PasswordLength = _MinimumPasswordLength
        End If
    End Sub

End Class