﻿
Imports System.IO

Class MainWindow

    Dim WithEvents ViewModel As MainWindowViewModel
    Dim PasswordFilePath As String = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "PasswordList", "Passwords.xml")
    Dim Passwords As New ObservableCollection(Of Password)
    Dim HasUnsavedChanges As Boolean = False

    Public Sub New()
        InitializeComponent()
        ViewModel = DirectCast(Me.DataContext, MainWindowViewModel)
        ViewModel.Passwords = New ViewModelCollection(Of Password, PasswordViewModel)(New ObservableCollectionSource(Of Password)(Passwords))
    End Sub

    Protected Overrides Sub OnContentRendered(e As System.EventArgs)
        MyBase.OnContentRendered(e)
        If File.Exists(PasswordFilePath) Then
            LoadFromFile()
        End If
    End Sub

    Protected Overrides Sub OnClosing(e As System.ComponentModel.CancelEventArgs)
        If HasUnsavedChanges AndAlso MessageBox.Show("Ungespeicherte Änderungen gehen verloren. Fortsetzen?", "Beenden?", MessageBoxButton.YesNo) = MessageBoxResult.No Then
            e.Cancel = True
        End If
        MyBase.OnClosing(e)
    End Sub

    Private Sub AddPassword(sender As System.Object, e As System.Windows.RoutedEventArgs)
        Dim Dlg As New PasswordDialog
        Dlg.Owner = Me
        Dlg.WindowStartupLocation = Windows.WindowStartupLocation.CenterOwner
        If Dlg.ShowDialog Then
            Passwords.Add(New Password With {.UserName = Dlg.UserName, .Password = Dlg.Password, .Description = Dlg.Description})
            HasUnsavedChanges = True
        End If
    End Sub

    Private Sub SavePasswords(sender As System.Object, e As System.Windows.RoutedEventArgs)
        Dim Dlg As New MainPasswordDialog With {.Repeat = True, .Owner = Me}
        Dlg.Owner = Me
        Dlg.WindowStartupLocation = Windows.WindowStartupLocation.CenterOwner
        If Dlg.ShowDialog Then
            FileAccess.PasswordFile.Save(PasswordFilePath, Dlg.Password, Passwords)
            HasUnsavedChanges = False
        End If
    End Sub

    Private Sub LoadPasswords(sender As System.Object, e As System.Windows.RoutedEventArgs)
        If Not HasUnsavedChanges OrElse MessageBox.Show("Ungespeicherte Änderungen gehen verloren. Fortsetzen?", "Überschreiben?", MessageBoxButton.YesNo) = MessageBoxResult.Yes Then
            LoadFromFile()
        End If
    End Sub

    Private Sub ViewModel_EditPassword(Sender As PasswordViewModel) Handles ViewModel.EditPassword
        Dim Dlg As New PasswordDialog
        Dlg.Owner = Me
        Dlg.WindowStartupLocation = Windows.WindowStartupLocation.CenterOwner
        Dlg.UserName = Sender.Target.UserName
        Dlg.Password = Sender.Target.Password
        Dlg.Description = Sender.Target.Description
        If Dlg.ShowDialog Then
            Sender.Target.UserName = Dlg.UserName
            Sender.Target.Password = Dlg.Password
            Sender.Target.Description = Dlg.Description
            HasUnsavedChanges = True
        End If
    End Sub

    Private Sub ViewModel_RemovePassword(Sender As PasswordViewModel) Handles ViewModel.RemovePassword
        If MessageBox.Show("Soll das Passwort wirklich gelöscht werden?", "Löschen?", MessageBoxButton.YesNo) = MessageBoxResult.Yes Then
            Passwords.Remove(Sender.Target)
            HasUnsavedChanges = True
        End If
    End Sub

    Private Sub LoadFromFile()
        If Not File.Exists(PasswordFilePath) Then
            Return
        End If
        Dim Dlg As New MainPasswordDialog
        Dlg.Owner = Me
        Dlg.WindowStartupLocation = Windows.WindowStartupLocation.CenterOwner
        Dlg.Repeat = False
        If Dlg.ShowDialog Then
            Dim Temp As IEnumerable(Of Password)
            Try
                Temp = FileAccess.PasswordFile.Load(PasswordFilePath, Dlg.Password)
            Catch ex As ProbablyInvalidPasswordException
                MessageBox.Show("Die Passwörter konnten nicht entschlüsselt werden. Wahrscheinlich ist das Passwort falsch.")
                Return
            End Try
            Passwords.Clear()
            For Each i In Temp
                Passwords.Add(i)
            Next
            HasUnsavedChanges = False
        End If
    End Sub

End Class
